#include <iostream>
#include <vector>
#include <map>
#include <cmath> // For std::abs
#include <algorithm> // For std::sort
#include <list>
// Struct to store trade data (price, volume, up/down direction)
struct Trade {
    double price;
    double volume;
    bool is_buy; // true = up volume (buy), false = down volume (sell)
};

// Class to calculate Volume Profile
class VolumeProfile {
public:
    // Constructor to initialize the bin size and number of rows
    VolumeProfile(double bin_size, double percentage = 0.7)
        : bin_size_(bin_size), percentage_(percentage) {}

    // Function to add trades to the profile
    void addTrade(double price, double volume, bool is_buy) {
        double price_bin = std::floor(price / bin_size_) * bin_size_;
        auto& volume_data = profile_[price_bin];
        if (is_buy) {
            volume_data.first += volume;  // Add to up volume
        } else {
            volume_data.second += volume; // Add to down volume
        }
    }

    // Function to generate the volume profile from a list of trades
    void generate(const std::vector<Trade>& trades) {
        for (const auto& trade : trades) {
            addTrade(trade.price, trade.volume, trade.is_buy);
        }
        calculatePOCAndValueArea();
    }

    // Function to display the volume profile
    void display() const {
        std::cout << "Price\tUp Volume\tDown Volume\n";
        for (const auto& [price_bin, volume_data] : profile_) {
            std::cout << "$" << price_bin << "\t" 
                      << volume_data.first << "\t\t" 
                      << volume_data.second << std::endl;
        }
    }

private:
    double bin_size_;   // Size of the price bins
    double percentage_; // Percentage for Value Area (default 70%)
    std::map<double, std::pair<double, double>> profile_; // Price bin mapped to (up_volume, down_volume)

    double total_volume_ = 0.0;
    double value_area_volume_ = 0.0;
    double poc_price_ = 0.0;
    double vah_ = 0.0;
    double val_ = 0.0;

    // Function to calculate POC (Point of Control) and Value Area
    void calculatePOCAndValueArea() {
        // Step 1: Calculate total volume
        total_volume_ = calculateTotalVolume();
        double target_value_area_volume = total_volume_ * percentage_;

        // Step 2: Sort profile by total volume (up + down)
        std::vector<std::pair<double, double>> sorted_profile;
        for (const auto& [price, volumes] : profile_) {
            double total_vol = volumes.first + volumes.second;
            sorted_profile.push_back({price, total_vol});
        }
        std::sort(sorted_profile.begin(), sorted_profile.end(), [](const auto& a, const auto& b) {
            return a.second > b.second; // Sort by total volume, descending
        });

        // Step 3: Find Point of Control (POC)
        poc_price_ = sorted_profile.front().first;
        value_area_volume_ = sorted_profile.front().second;

        // Step 4 & 5: Expanding Value Area by adding rows above and below POC
        size_t poc_index = 0;
        for (size_t i = 0; i < sorted_profile.size(); ++i) {
            if (std::abs(sorted_profile[i].first - poc_price_) < 1e-6) {
                poc_index = i;
                break;
            }
        }

        size_t upper_index = poc_index + 1;
        size_t lower_index = poc_index - 1;

        while (value_area_volume_ < target_value_area_volume) {
            double upper_volume = (upper_index < sorted_profile.size()) ? sorted_profile[upper_index].second : 0;
            double lower_volume = (lower_index >= 0) ? sorted_profile[lower_index].second : 0;

            if (upper_volume > lower_volume) {
                value_area_volume_ += upper_volume;
                vah_ = sorted_profile[upper_index].first;
                upper_index++;
            } else {
                value_area_volume_ += lower_volume;
                val_ = sorted_profile[lower_index].first;
                if (lower_index > 0) lower_index--;
            }
        }
    }

    // Helper function to calculate total volume in the profile
    double calculateTotalVolume() const {
        double total_volume = 0.0;
        for (const auto& [price_bin, volume_data] : profile_) {
            total_volume += (volume_data.first + volume_data.second);
        }
        return total_volume;
    }
};

class TickLevel {
 public:
   using OrderIds = std::vector<int64_t>;
   TickLevel(int64_t ticks) noexcept:
    ticks_{ticks}
   {
   }
    
   ~TickLevel() noexcept = default;
   
   // Copy.
   TickLevel(const TickLevel&) = delete;
   TickLevel& operator=(const TickLevel&) = delete;
   
   // Move.
   TickLevel(TickLevel&&) = default;
   TickLevel& operator=(TickLevel&&) = delete;

   inline auto ticks () const { return ticks_; }
   inline auto& ids() { return ids_; }
   inline const auto& ids() const { return ids_; }
   void push(int64_t id) { ids_.push_back(id); }

 private:
   const int64_t ticks_;
   OrderIds ids_;
};


int main() {
	std::list<TickLevel> l;
    std::cout << l.front().ticks() << std::endl;

    return 0;
}

